\ Definition for the forth vocabulary
" forth" constant forth

: print-iterator ( obj -- )
    begin
        dup " hasNext" jinvoke0 jbool>int not if leave then
        dup " next" jinvoke0 type space
    repeat drop ;

: words ( voc -- )
    current-vocabulary " getWords" jinvoke0 
    print-iterator ;

: vocs ( -- )
    current-dictionary " getVocs" jinvoke0
    print-iterator ;

: (get-immediate-flag) ( word -- 1 | -1 )
    current-dictionary " isImmediate" jinvoke1 jbool>int
    if 1 else -1 then ;

: (get-xt) ( word -- xt )
    current-dictionary " getExecutionToken" jinvoke1 ;

: (get-xt-flag) ( word -- xt 1  |  xt -1 )
    dup (get-immediate-flag)    \ word flag
    swap (get-xt)               \ flag xt
    swap ;

: (hasword) ( word -- 0|-1) 
    current-dictionary " hasWord" jinvoke1 jbool>int ;

: find ( word -- word 0  |  xt 1  |  xt -1 )
    dup (hasword)
    if (get-xt-flag)
    else 0 then ;

: (ensurevoc) ( vocname -- )
    current-dictionary " ensureVoc" jinvoke1 drop ;

: (forthvocabulary) ( -- forthvoc )
    forth current-dictionary " getVoc" jinvoke1 ;

: (createvocword) ( vocname -- ) 
    dup here jfalse (forthvocabulary)  " registerWordFromForth" 3 jinvoken drop
    " leonardoce.forth.vm.primitives.StaticPushData" 1 jnewn ,
    " leonardoce.forth.vm.primitives.ReturnPrimitive" 0 jnewn , ;    

: vocabulary ( <vocname> -- )
    word 
    dup (ensurevoc)
    dup (hasword) not if (createvocword) else drop then ;
    
: definitions ( vocname -- )
    current-dictionary " getVoc" jinvoke1 
    current-dictionary " setActiveVocabulary" jinvoke1
    drop ;

: (resetsearchvocs) ( -- )
    current-dictionary " resetSearchVocs" jinvoke0 drop ;

: (addsearchvoc) ( voc -- )
    dup " forth" string= not if 
        current-dictionary " addSearchVoc" jinvoke1 drop
    else 
        drop
    then ;

: only ( <vocname> -- )
    word  
    (resetsearchvocs)
    (addsearchvoc) ;

: also ( <vocname> -- )
    word (addsearchvoc) ;

: order ( -- )
    current-dictionary " getOrder" jinvoke0 print-iterator
    current-vocabulary . cr ;

\ -----------------------------------
\ --------------------- Documentation
\ -----------------------------------

" print-iterator" doc"
( iter -- )
Print the contents of this iterator "

" vocs" doc"
( -- )
List the vocabularies available "

" words" doc"
( -- )
List the words available in the current vocabulary "

" find" doc"
( word -- word 0  |  xt 1  |  xt -1 )
Find the definition named in the word. 
If the definition is not found, return word and zero. 
If the definition is found, return its execution token xt. 
If the definition is immediate, also return one (1), 
otherwise also return minus-one (-1). 
For a given string, the values returned by FIND while 
compiling may differ from those returned while not compiling. 
(see ANS94) "

" vocabulary" doc"
( <word> -- )
Create, if not already present, a new vocabulary "

" definitions" doc"
( vocname -- )
Change the active vocabulary "

" only" doc"
( <word> -- )
Reset the vocabulary search path then appends to it the vocabulary passed "

" also" doc"
( <word> -- )
Append to the vocabulary search path the vocabulary passed "

" order" doc"
( -- )
Print the current vocabulary search path and the active vocabulary
name "