/*
 * BootVmStatus.java
 *
 * Created on 17 agosto 2007, 21.40
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package leonardoce.forth.boot;

import leonardoce.forth.ForthException;
import leonardoce.forth.LeoForth;
import leonardoce.forth.vm.DefaultVmStatus;
import leonardoce.forth.vm.InputStreamAdaptor;
import leonardoce.forth.vm.Primitive;
import leonardoce.forth.vm.primitives.cflow.*;
import leonardoce.forth.vm.primitives.obj.*;
import leonardoce.forth.vm.primitives.parsing.*;
import leonardoce.forth.vm.primitives.sys.*;
import leonardoce.forth.vm.primitives.doublemath.*;
import leonardoce.forth.vm.primitives.returnstack.*;
import leonardoce.forth.vm.primitives.math.*;
import leonardoce.forth.vm.primitives.memory.*;
import leonardoce.forth.vm.primitives.datastack.*;
import leonardoce.forth.vm.primitives.ReturnPrimitive;

/**
 *
 * @author leonardo
 */
public class BootVmStatus extends DefaultVmStatus {
    public BootVmStatus() {
        super();
        
        registerPrimitives();
        
        // Load boot forth code        
        java.io.InputStream is=getClass().getClassLoader().getResourceAsStream("leonardoce/forth/boot/boot.fth");
        setInputStream(new InputStreamAdaptor(is));
        LeoForth lf=new LeoForth(this);
        try {
            lf.run();
        } catch(ForthException fe) {
            fe.printStackTrace();
        }
        
        // Load forth utilities
        is=getClass().getClassLoader().getResourceAsStream("leonardoce/forth/boot/utils.fth");
        setInputStream(new InputStreamAdaptor(is));
        try {
            lf.run();
        } catch(ForthException fe) {
            fe.printStackTrace();
        }
        
        // Load docsys utilities
        is=getClass().getClassLoader().getResourceAsStream("leonardoce/forth/boot/docsys.fth");
        setInputStream(new InputStreamAdaptor(is));
        try {
            lf.run();
        } catch(ForthException fe) {
            fe.printStackTrace();
        }
        
        // Load docsys utilities
        is=getClass().getClassLoader().getResourceAsStream("leonardoce/forth/boot/documentation.fth");
        setInputStream(new InputStreamAdaptor(is));
        try {
            lf.run();
        } catch(ForthException fe) {
            fe.printStackTrace();
        }
        
        // Load vocabulary utilities
        is=getClass().getClassLoader().getResourceAsStream("leonardoce/forth/boot/vocabularies.fth");
        setInputStream(new InputStreamAdaptor(is));
        try {
            lf.run();
        } catch(ForthException fe) {
            fe.printStackTrace();
        }
        
        setInputStream(null);
    }

    private void registerPrimitives() {
        
        // System
        registerPrimitive(new Colon());
        registerPrimitive(new ColonI());
        registerPrimitive(new ToRet());
        registerPrimitive(new FromRet());
        registerPrimitive(new Semicolon());
        registerPrimitive(new Create());
        registerPrimitive(new Does());
        registerPrimitive(new Here());
        registerPrimitive(new HereSet());
        registerPrimitive(new Allot());
        registerPrimitive(new CharPrim());
        registerPrimitive(new BracketTick());
        registerPrimitive(new Tick());
        registerPrimitive(new Execute());
        registerPrimitive(new PostPone());
        registerPrimitive(new CompileComma());
        registerPrimitive(new Literal());
        registerPrimitive(new Compiling());
        registerPrimitive(new VmStatusPrim());
        
        // Objs
        registerPrimitive(new ToClassName());
        registerPrimitive(new NullQuestion());
        registerPrimitive(new ParenDot());
        registerPrimitive(new JNewN());
        registerPrimitive(new JInvokeN());
        
        // Parsing
        registerPrimitive(new Parse());
        registerPrimitive(new Word());
        
        // Memory management
        registerPrimitive(new Load());
        registerPrimitive(new Store());
        
        // Control flow
        registerPrimitive(new IfPrim());
        registerPrimitive(new ThenPrim());
        registerPrimitive(new ElsePrim());
        registerPrimitive(new BeginPrim());
        registerPrimitive(new RepeatPrim());
        registerPrimitive(new LeavePrim());
        
        // Math
        registerPrimitive(new Plus());
        registerPrimitive(new Minus());
        registerPrimitive(new Divide());
        registerPrimitive(new Multiply());
        registerPrimitive(new Mod());
        registerPrimitive(new IToD());
        registerPrimitive(new AndPrim());
        registerPrimitive(new NotPrim());
        registerPrimitive(new OrPrim());
        registerPrimitive(new Greater());
        registerPrimitive(new Lesser());
        registerPrimitive(new ToCharString());
        
        // Data Stack
        registerPrimitive(new Length());
        registerPrimitive(new Dup());
        registerPrimitive(new Rot());
        registerPrimitive(new Swap());
        registerPrimitive(new Drop());
        registerPrimitive(new Dot());
        registerPrimitive(new DotS());
        
        // Double math
        registerPrimitive(new DEqual());
        registerPrimitive(new DPlus());
        registerPrimitive(new DMinus());
        registerPrimitive(new DDivide());
        registerPrimitive(new DMultiply());
        registerPrimitive(new DMod());
        registerPrimitive(new DCos());
        registerPrimitive(new DSin());
        registerPrimitive(new DTan());
        registerPrimitive(new DExp());
        registerPrimitive(new DTruncate());
        registerPrimitive(new DRound());
        registerPrimitive(new DGreater());
        registerPrimitive(new DLesser());
    }
    
    public void registerPrimitive(Primitive p)
    {
        try {
            getCurrentMemory().store(getHere(), new Comment("Primitive \""+p.getName()+"\""));
            getCurrentMemory().store(getHere()+1, p);
            getCurrentMemory().store(getHere()+2, new ReturnPrimitive());
            getCurrentDictionary().getActiveVocabulary().registerWord(p.getName(), getHere(), p.isImmediate());
            getCurrentDictionary().getActiveVocabulary().setDocumentation(p.getName(), p.getDocumentation());
            setHere(getHere()+3);
        } catch(ForthException fe) {
            fe.printStackTrace();
        }
    }
}
