/*
 * SwingForthEditor.java
 *
 * Created on 20 agosto 2007, 16.51
 */

package leonardoce.forth.gui.swing;

import java.io.File;
import java.util.Iterator;
import java.util.Vector;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import leonardoce.forth.gui.ForthEditorController;
import leonardoce.forth.gui.ForthEditorView;

/**
 *
 * @author  leonardo
 */
public class SwingForthEditor extends javax.swing.JPanel 
        implements ForthEditorView, DocumentListener {
    private ForthEditorController controller;
    
    /** Creates new form SwingForthEditor */
    public SwingForthEditor() {
        initComponents();
    }

    public ForthEditorController getController() {
        return controller;
    }

    public void setController(ForthEditorController controller) {
        this.controller = controller;
    }

    public String getText() {
        return getSyntaxPane().getText();
    }

    public void showErrorMessage(String msg) {
        JOptionPane.showMessageDialog(this, msg, "Error", JOptionPane.ERROR_MESSAGE);
    }

    public void setText(String s) {
        getSyntaxPane().setText(s);
    }
    
    public void requestFocus() {
        getSyntaxPane().requestFocus();
    }

    public leonardoce.forth.gui.swing.ForthSyntaxPane getSyntaxPane() {
        InternalTab it=(InternalTab)tabbedPane.getSelectedComponent();
        if(it==null) {
            return null;
        } else {
            return it.getSyntaxPane();
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        tabbedPane = new javax.swing.JTabbedPane();

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(tabbedPane, javax.swing.GroupLayout.DEFAULT_SIZE, 525, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(tabbedPane, javax.swing.GroupLayout.DEFAULT_SIZE, 515, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents

    public void setCurrentTabTitle(String s) {
        int idx=tabbedPane.getSelectedIndex();
        if(idx!=-1) {
            tabbedPane.setTitleAt(idx, s);
        }
    }

    public void setCurrentTabFile(File f) {
        int idx=tabbedPane.getSelectedIndex();
        if(idx!=-1) {
            InternalTab it=(InternalTab)tabbedPane.getComponentAt(idx);
            it.setFile(f);
        }
    }

    public File getCurrentTabFile() {
        int idx=tabbedPane.getSelectedIndex();
        if(idx!=-1) {
            InternalTab it=(InternalTab)tabbedPane.getComponentAt(idx);
            return it.getFile();
        } else {
            return null;
        }
    }

    public void createNewTab() {
        InternalTab sp=new InternalTab();
        tabbedPane.addTab("", sp);
        tabbedPane.setSelectedIndex(tabbedPane.getTabCount()-1);
    }

    public void deleteActiveTab() {
        int idx=tabbedPane.getSelectedIndex();
        if(idx!=-1) {
            tabbedPane.remove(idx);
        }
    }

    public String getCurrentTabTitle() {
        int idx=tabbedPane.getSelectedIndex();
        if(idx!=-1) {
            return tabbedPane.getTitleAt(idx);
        } else {
            return "";
        }
    }

    public void insertUpdate(DocumentEvent e) {
        controller.signalChanged();
    }

    public void removeUpdate(DocumentEvent e) {
        controller.signalChanged();
    }

    public void changedUpdate(DocumentEvent e) {
        // No op. We don't mind style changes'
    }

    public Iterable<String> getTabTitles() {
        Vector<String> v=new Vector<String>();
        for(int i=0;i<tabbedPane.getTabCount();i++) {
            v.addElement(tabbedPane.getTitleAt(i));
        }
        return v;
    }

    private class InternalTab extends JScrollPane {
        ForthSyntaxPane sp;
        private File file;
        
        public InternalTab() {
            super();
            sp=new ForthSyntaxPane();
            setViewportView(sp);
            sp.getDocument().addDocumentListener(SwingForthEditor.this);
        }
        
        public ForthSyntaxPane getSyntaxPane() {            
            return sp;
        }

        public File getFile() {
            return file;
        }

        public void setFile(File file) {
            this.file = file;
        }
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTabbedPane tabbedPane;
    // End of variables declaration//GEN-END:variables
    
}
