/*
 * Vocabulary.java
 *
 * Created on 17 agosto 2007, 18.36
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package leonardoce.forth.vm;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;
import leonardoce.forth.ForthException;

/**
 *
 * @author leonardo
 */
public class Dictionary {
    private Hashtable<String, Vocabulary> vocs;
    private Vector<Vocabulary> searchVocs;
    private Vocabulary activeVoc;

    public Dictionary() {
        vocs=new Hashtable<String, Vocabulary>();
        activeVoc=new Vocabulary("forth");
        vocs.put("forth", activeVoc);
        searchVocs=new Vector<Vocabulary>();
        searchVocs.add(activeVoc);
    }
    
    private Vocabulary getVocabularyForWord(String wordName) {
        if(activeVoc.hasWord(wordName)) {
            return activeVoc;
        }
        
        for(Vocabulary v: searchVocs) {
            if(v.hasWord(wordName)) {
                return v;
            }
        }
        
        return null;
    }
    
    public int getExecutionToken(String wordName) throws ForthException
    {
        Vocabulary v=getVocabularyForWord(wordName);
        if(v==null) {
            throw new ForthException("Unknown word "+wordName);
        } else {
            return v.getExecutionToken(wordName);
        }
    }
    
    public boolean isImmediate(String wordName) {
        return getVocabularyForWord(wordName).isImmediate(wordName);
    }
    
    public boolean hasWord(String word) {
        Vocabulary v=getVocabularyForWord(word);
        if(v==null) {
            return false;
        } else {
            return true;
        }
    }
    
    public String dump() {
        StringBuffer sb=new StringBuffer();
        for(String key: vocs.keySet())
        {
            sb.append(key);
            Vocabulary vb=vocs.get(key);
            sb.append(vb.dump());
        }
        return sb.toString();
    }
    
    public Vocabulary getActiveVocabulary() {
        return activeVoc;
    }
    
    public void setActiveVocabulary(Vocabulary v) {
        activeVoc=v;
    }
    
    public void resetSearchVocs() {
        searchVocs.clear();
        searchVocs.add(vocs.get("forth"));
    }
    
    public void addSearchVoc(String name) {
        searchVocs.add(vocs.get(name));
    }

    public Iterator<Vocabulary> getOrder() {
        return searchVocs.iterator();
    }    
    
    public boolean hasVoc(String name) {
        return vocs.containsKey(name);
    }
    
    public void ensureVoc(String name) {
        if(!hasVoc(name)) {
            vocs.put(name, new Vocabulary(name));
        }
    }
    
    public Vocabulary getVoc(String name) {
        return vocs.get(name);
    }
    
    public Iterator<String> getVocs() {
        return vocs.keySet().iterator();
    }
}
