/*
 * Vocabulary.java
 *
 * Created on 17 agosto 2007, 18.36
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package leonardoce.forth.vm;

import java.util.Hashtable;
import java.util.Iterator;
import leonardoce.forth.ForthException;
import leonardoce.forth.UnknownWordException;

/**
 *
 * @author leonardo
 */
public class Vocabulary {
    private Hashtable<String, VocabRecord> table;
    private String name;
    
    /** Creates a new instance of Vocabulary */
    public Vocabulary(String name) {
        table=new Hashtable<String, VocabRecord>();
        this.name=name;
    }
    
    public int getExecutionToken(String wordName)
    {
        return table.get(wordName).xt;
    }
    
    public void registerWord(String wordName, int executionToken, boolean immediate)
    {
        table.put(wordName, new VocabRecord(immediate, executionToken));
    }

    public void registerWordFromForth(String wordName, Integer executionToken, Boolean immediate) {
        registerWord(wordName, executionToken, immediate);
    }
    
    
    public boolean isImmediate(String wordName) {
        return table.get(wordName).immediate;
    }
    
    public boolean hasWord(String word) {
        return table.containsKey(word);
    }
    
    public void setDocumentation(String word, String documentation) throws ForthException {
        if(!hasWord(word)) {
            throw new UnknownWordException(word);
        }
        table.get(word).documentation=documentation;
    }
    
    public String getDocumentation(String word) throws ForthException {
        if(!hasWord(word)) {
            throw new UnknownWordException(word);
        }
        return table.get(word).documentation;
    }
    
    public Iterator<String> getWords() {
        return table.keySet().iterator();
    }
    
    private static class VocabRecord {
        public boolean immediate;
        public int xt;
        public String documentation;
        
        public VocabRecord(boolean p_im, int p_xt) {
            immediate=p_im;
            xt=p_xt;
            documentation="";
        }                
    }
    
    public String dump() {
        StringBuffer sb=new StringBuffer();
        for(String key: table.keySet())
        {
            sb.append(key);
            VocabRecord vb=table.get(key);
            sb.append(" ");
            if(vb.immediate) {
                sb.append("Immediate ");
            }
            sb.append("(");
            sb.append(vb.xt);
            sb.append(")");
            sb.append("\n");
        }
        return sb.toString();
    }
    
    public String getName() {
        return name;
    }
    
    public String toString() {
        return name;
    }
}
