/*
 * Minus.java
 *
 * Created on 17 agosto 2007, 22.46
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package leonardoce.forth.vm.primitives.obj;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import leonardoce.forth.ForthException;
import leonardoce.forth.vm.AbstractPrimitive;
import leonardoce.forth.vm.VmStatus;

/**
 *
 * @author leonardo
 */
public class JInvokeN extends AbstractPrimitive {
    
    /** Creates a new instance of Plus */
    public JInvokeN() {
    }

    private static boolean isPrivateOrProtected(Class<?>t) {
        return ((t.getModifiers() & java.lang.reflect.Modifier.PRIVATE)!=0 || 
                (t.getModifiers() & java.lang.reflect.Modifier.PROTECTED)!=0);
    }
    
    private static boolean classHasMethod(Class<?>t, String name, Class<?>[] args) {
        try {
            t.getMethod(name,args);
            return true;
        } catch (SecurityException ex) {
            ex.printStackTrace();
            return false;
        } catch (NoSuchMethodException ex) {
            return false;
        }
    }
    
    public void execute(VmStatus is) throws ForthException {
        Integer nArgs=(Integer)is.getDataStack().pop();
        String methodName=(String)is.getDataStack().pop();
        Object destinationObj=(Object)is.getDataStack().pop();
        
        if(destinationObj==null) {
            throw new ForthException("Can't invoke method on null object");
        }
        
        Class<?>[] argsClasses=new Class<?>[nArgs];
        Object[] args=new Object[nArgs];
        
        for(int i=args.length-1;i>=0;i--) {
            args[i]=is.getDataStack().pop();
            if(args[i]==null) {
                argsClasses[i]=Object.class;
            } else {
                argsClasses[i]=args[i].getClass();
            }
        }
        
        try {
            Class<?> t=destinationObj.getClass();
            
            Method m=null;
            try {
                // Try an exact match
                m=t.getMethod(methodName, argsClasses);
            } catch(NoSuchMethodException nsm) {
                // Find a method with this name and the same formal parameters count
                Method[] methods=t.getMethods();
                for(int i=0;i<methods.length;i++) {
                    if(methods[i].getName().equals(methodName) &&
                            methods[i].getParameterTypes().length==nArgs) {
                        m=methods[i];
                        break;
                    }
                }

                if(m==null) {
                    throw new ForthException("Can't find method name \""+methodName+"\" in class \""+t+"\" for \""+nArgs+" formal parameters.");
                } else {
                    for(int i=0;i<m.getParameterTypes().length;i++) {
                        argsClasses[i]=m.getParameterTypes()[i];
                    }
                }
            }
            
            // A fix bug java bug #4071957
            while(isPrivateOrProtected(t) && !t.equals(Object.class)) {
                // It's not ok but I will try to find the superclass or interface that has the requested method
                if(classHasMethod(t.getSuperclass(), methodName, argsClasses)) {
                    t=t.getSuperclass();
                } else {
                    Class<?>[] interf=t.getInterfaces();
                    for(int i=0;i<interf.length;i++) {
                        if(classHasMethod(interf[i], methodName, argsClasses)) {
                            t=interf[i];
                            m=t.getMethod(methodName, argsClasses);
                            break;
                        }
                    }
                }
            }
                
            is.getDataStack().push(m.invoke(destinationObj, args));
        } catch(InvocationTargetException e) {
            throw new ForthException(e.getTargetException());
        } catch(Exception e) {
            throw new ForthException(e);
        } 
        
    }
    
    public String getName()
    {
        return "jinvoken";
    }  

    public String getDocumentation() {
        return "( arg1...argn destobj methodName n-args -- obj )\n"+
                "Invoke a java method using the arguments passed.\n"+
                "The method will be invocated on the destination object passed.";
    }
}
